#include <stdio.h>
#include <ME218_C32.h>
#include <timers12.h>
#include <string.h>
#include "PWMS12.h"
#include "ADS12.h"

#define command_mode 0
#define data_mode 1

#define PortA 0
#define PortB 1
#define PortC 2

// Reserved variable for 7 segments LED Display
#define ZERO 0x01  //0 000 0001
#define ONE 0x4F   //0 100 1111
#define TWO 0x12   //0 001 0010
#define THREE 0x06 //0 000 0110
#define FOUR 0x4C  //0 100 1100
#define FIVE 0x24  //0 010 0100
#define SIX 0x20   //0 010 0000
#define SEVEN 0x0D //0 000 1101
#define EIGHT 0x00 //0 000 0000
#define NINE 0x04  //0 000 0100
#define OFF 0x7F  //0 111 1111


#define RESET_STATE 0
#define COIN_STATE 1
#define LOW_PLAY_STATE 2
#define MED_PLAY_STATE 3
#define HIGH_PLAY_STATE 4
#define FAILURE_STATE 5
#define SUCCESS_STATE 6

#define COIN 0
#define START_SW 1
#define WireTouch 2

static int LimitTime = 0;
static int current_region = 1;
static int LifeNumber = 0;
static int solenoid_onTime = 0;
static int solenoid_onCommand = 0;
static int CurrentCoinNumber = 0;
static int StartSignal = 0;
static int FailureSignal = 0;

static unsigned char Displayed_data[20];
static unsigned char current_PortC_value = 0x00;
static unsigned char current_extention2_addr = 0x00;

static void delay_ms_times(int millisec);
static void delay_with_service(int millisec);
static void PORTInit(void);
static void control_LCD(int select ,unsigned char contents);
static void delay(int count);
static void LCDputString(unsigned char column, unsigned char* CharacterString);

								 
static int GetSignal(unsigned char pin);
static void DataSend(unsigned char portNumber, unsigned char data);
static unsigned char DataReceive(void);
static void ExtensionChipInit(void);				 
static void DataSend2(unsigned char portNumber, unsigned char data);
static void Motor_Vibrator_Control(int motor_mode, int motor_speed, int vibrator_on);

static void LED_TimeDisplay(unsigned char sec);
static void LED_LevelDisplay(unsigned int level);
static void LED_LifeDisplay(unsigned int lifenumber);
static void warming_up(void);

static void CoinDetecting(void);
static void StartDetecting(void);
static void WireDetecting(void);
static void SWAG_dispense(int duration);
static void EndingSound(int mode);


// Port Initialization of C32, P8255(port extension chip)
static void PORTInit(void)
{	 
  int i;
  
  DDRM = 0xFF;  //set PortM as outputs
  DDRT = 0x1F;  // 0001 1111 set 7,6,5 bits PortT as inputs
  DDRAD = 0x7F;   // Initially all outputs for data sending (PORT AD) 
  while(ADS12_Init("AOOOOOOO") != ADS12_OK)
    PTM = PTM;
  
  puts("ADC Port initialization!");
  
  //DDRAD = 0x7F; 
  PTT = 0x00;		// initial values
  PTM = 0x10;   // initial values
  PTAD = 0x00;   // initial values
  
  PWMS12_Init();		// Initialization of PWM Port
  PWMS12_SetDuty(0, PWMS12_CHAN0);
  PWMS12_SetPeriod(PWMS12_4000US, PWMS12_GRP1); // set PWM period of channel 2    
  PWMS12_SetDuty(23, PWMS12_CHAN2);
  ExtensionChipInit();
  
  // Displayed data array initilization
  for(i=0;i<20;i++)
  {
      Displayed_data[i] = 0x20 ;
  }
  
  StartSignal = 0;
  FailureSignal = 0;
  
  
}


// Port Initialization of P8255(port extension chip)
static void ExtensionChipInit(void)
{
        PTM = PTM | 0x10;   // 0001 0000, (not)cs = 1 
        PTM = PTM | 0x08;   // 0000 1000, (not)RD = 1
        delay_ms_times(1);
        
        //-------------------------------------------
        PTAD = 0x81 ;    //control word
        PTM = PTM | 0x20;   // control word
        //-------------------------------------------
        
        PTM = PTM | 0x03;   // 0000 0011, A1 A0: 1 1
        delay_ms_times(1);
        
        PTM = PTM & 0xEF;   // 1110 11110, (not)cs = 0 
			                
        delay_ms_times(1);        
        
        PTM = PTM & 0xFB;   // 1111 1011, (not)WR = 0
        delay_ms_times(1);
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
        delay_ms_times(1);
        PTM = PTM | 0x10;   // 0001 0000, (not)cs = 1 
        
        // From this, initial of LED's extension chip( The 2nd P8255 )
               
        DataSend(PortB, 0x80); // (not)WR = 1
        delay_ms_times(1);
        
        current_extention2_addr = 0xC0; // A1 A0: 1 1
        DataSend(PortC, 0x00);
        
        delay_ms_times(1);
               
        DataSend(PortB, 0x00); // (not)WR = 0
        delay_ms_times(2);
        DataSend(PortB, 0x80); // (not)WR = 1
}


// Initialization of a LCD module 
void LCDInit(void)
{
  PORTInit();

  puts("Inititalization Start!");
  control_LCD(command_mode, 0x30);
  delay_ms_times(5);
  
  
  control_LCD(command_mode, 0x30);
  delay_ms_times(1);
  
  control_LCD(command_mode, 0x30);
  delay_ms_times(2);
  
  control_LCD(command_mode, 0x08);
  control_LCD(command_mode, 0x01);
  delay_ms_times(5);
  
  control_LCD(command_mode, 0x06);
  control_LCD(command_mode, 0x0F);
  puts("Inititalization End!"); 
  
}

// A sub function used for LCD initialization
static void control_LCD(int select ,unsigned char contents)
{
  if(select == 0)  // command mode
  {
  	//PTT = contents;  //apply data
  	PTT = PTT & 0xEF;  // and with 1110 1111. Set EN low(PortT4 low)
  	DataSend(PortA, contents);
  	delay(2); 
    PTT = PTT & 0xF7;  // and with 1111 0111. Set RS 0(PortT3 low)
    delay(2); 
    PTT = PTT | 0x10;  // or with 0001 0000. Set EN high(PortT4 high)    
    delay(2); 
    PTT = PTT & 0xEF;  // and with 1110 1111. Set EN low(PortT4 low)    
  }
  else if(select ==1)  // data mode
  {
    //PTT = contents;  //apply data
    PTT = PTT & 0xEF;  // and with 1110 1111. Set EN low(PortT4 low)
    DataSend(PortA, contents);
    delay(2); 
    PTT = PTT | 0x08;  // or with 0000 1000. Set RS 1(PortT3 high)
    delay(2); 
    PTT = PTT | 0x10;  // or with 0001 0000. Set EN high(PortT4 high)    
    delay(2); 
    PTT = PTT & 0xEF;  // and with 1110 1111. Set EN low(PortT4 low)  
  }
  else
  { 
    ; //NOP
  }
}



// A function which allows to print a character on the LCD.
void LCDputchar(unsigned char one_character)
{
   int i; 
   
     for(i=0; i<19; i++)
	 {
	    Displayed_data[i] = Displayed_data[i+1];
	 }
	 
	 Displayed_data[19] = one_character;
	 
	 control_LCD(command_mode, 0x80);		// print at the leftmost position
	 
	 for(i=0; i<20; i++)
	 {							  
	    control_LCD(data_mode, Displayed_data[i]);
   }
   
   control_LCD(command_mode, 0x10);		// move the cursor one position left.
   
}


// A function which allows to print a string, combination of characters on the LCD.
void LCDputString(unsigned char column, unsigned char* CharacterString)
{
   int i; 
   
   
   
   for(i=0; i<20; i++)
	 {
	    if(i<column)
	        Displayed_data[i] = ' ';
	    else
	        Displayed_data[i] = CharacterString[i-column];
	 }
	 
	 control_LCD(command_mode, 0x80);		// print at the leftmost position	 
	 
	 for(i=0; i<20; i++)
	 {							  
	    control_LCD(data_mode, Displayed_data[i]);
     }
     
}

// A naive delay function for not accurate and simple delay. 
static void delay(int count)
{
	int j, k;
	
	for(j = 0; j<count; j++)	// do nothing but only killing time
	  for(k = 0; k<10 ; k++)
	    PTM = PTM & 0xff;
}


// Accurate delay function. Delay resolution is a millisecond.
static void delay_ms_times(int millisec)
{
  int start,current;
  
  TMRS12_Init(TMRS12_RATE_1MS);   
  start = TMRS12_GetTime(); 
  current = TMRS12_GetTime();
  
  while( (current-start) <= millisec)
  {
    current = TMRS12_GetTime();
  }
}


// 'Accurate delay + event checking' function.
static void delay_with_service(int millisec)
{
  int start,current;
  int flag=0;
  int i;
  int j=0;
  
  TMRS12_Init(TMRS12_RATE_1MS);   
  start = TMRS12_GetTime(); 
  current = TMRS12_GetTime();
  
  while( (current-start) <= millisec )
  {
    
    ////////////////////////////////////////////////////////////////
    if((DataReceive() & 0x02) == 0x02)   // 1st passing point
    {
                current_region = 2;
                solenoid_onCommand = 1;  // turn on the solenoid!!!
                solenoid_onTime = 1; // give 5 seconds
                //solenoid on for some time.
                 PWMS12_SetDuty(50, PWMS12_CHAN1);   // buzzer on
                delay_ms_times(200);
                PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	            
                
                break;
    
    }
    else if((DataReceive() & 0x04) == 0x04)   // 3rd passing point
    {
           current_region = 3;
            PWMS12_SetDuty(50, PWMS12_CHAN1);   // buzzer on
                delay_ms_times(200);
                PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off
           break;
    }
    else if((DataReceive() & 0x08) == 0x08)   // 4th passing point
    {
          for(i=0; i<10;i++)
          {
            if((DataReceive() & 0x08) == 0x08)
                j++;
          }
          
          if(j>8)
          {
            current_region = 4;
             PWMS12_SetDuty(50, PWMS12_CHAN1);   // buzzer on
                delay_ms_times(200);
                PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off
            break;
          }
    }
    else
        current_region = current_region;
    /////////////////////////////////////////////////////////////////
      
    if( GetSignal(WireTouch) == 1 ) // wire touched!!
    {
        while( GetSignal(WireTouch) == 1 );
               
        LifeNumber--;	// decrease the number of life.
        LED_LifeDisplay(LifeNumber);
        
        PWMS12_SetDuty(70, PWMS12_CHAN1);   // buzzer on
        delay_ms_times(500);			// for 500 ms
        PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off
                
        if(LifeNumber == -1)
            FailureSignal = 1;
    }
    
    current = TMRS12_GetTime();    
    
  }
  
   PTM = PTM | 0x10; // 0001 0000 Turn off the solenoid. 
    
    if( (solenoid_onCommand == 1) && (solenoid_onTime >0))
    {
        // solenoid on!
        solenoid_onTime--;
        PTM = PTM & 0xEF; // 1110 1111 Turn on the solenoid.         
    }
    
    if(solenoid_onTime == 0)	// when the solenoid-on-time ends, 
         solenoid_onCommand = 0; // turn off the solenoid
  
}


// A function which check the input signals of the coin detector and the start switch.
static int GetSignal(unsigned char pin)
{
    if(pin == COIN)
    {           
          if( (PTT & 0x20) == 0x20)	// When COIN interferes the coin detecting sensor
            return 1;  			// Coin detected
          else
            return 0;			// NO COIN
    }
    else if(pin == START_SW)		
    {           
          if( (PTT & 0x40) == 0x40)	// When the game start button is pushed
            return 1;  
          else
            return 0;			// Still not pushed
    }
    else
    {           
          if( (PTT & 0x80) == 0x80)	// When a player touches the wire
            return 1;  
          else
            return 0;			// Not touched
    }
}
 

// A function which convert the input argument(Game time) to the proper signals so that the 7 segment LED displays the Game time.
static void LED_TimeDisplay(unsigned char sec)
{
    unsigned char upper_digit, lower_digit;
    
    lower_digit = sec%10;
    upper_digit = sec-lower_digit;
    
    if(lower_digit == 0)
        DataSend2(PortB, ZERO);
    else if(lower_digit == 1)
        DataSend2(PortB, ONE);
    else if(lower_digit == 2)
        DataSend2(PortB, TWO);
    else if(lower_digit == 3)
        DataSend2(PortB, THREE);
    else if(lower_digit == 4)
        DataSend2(PortB, FOUR);
    else if(lower_digit == 5)
        DataSend2(PortB, FIVE);
    else if(lower_digit == 6)
        DataSend2(PortB, SIX);
    else if(lower_digit == 7)
        DataSend2(PortB, SEVEN);
    else if(lower_digit == 8)
        DataSend2(PortB, EIGHT);
    else
        DataSend2(PortB, NINE);
    
    
    if(upper_digit == 0)
        DataSend2(PortA, OFF);
    else if(upper_digit == 10)
        DataSend2(PortA, ONE);
    else if(upper_digit == 20)
        DataSend2(PortA, TWO);
    else if(upper_digit == 30)
        DataSend2(PortA, THREE);
    else if(upper_digit == 40)
        DataSend2(PortA, FOUR);
    else if(upper_digit == 50)
        DataSend2(PortA, FIVE);
    else if(upper_digit == 60)
        DataSend2(PortA, SIX);
    else if(upper_digit == 70)
        DataSend2(PortA, SEVEN);
    else if(upper_digit == 80)
        DataSend2(PortA, EIGHT);
    else
        DataSend2(PortA, NINE);
    
}
       
       
// A function which displays the game level that the player currently chooses on the 7 segment LED.       
static void LED_LevelDisplay(unsigned int level)
{
    
   if(level == 1)
   	  DataSend2(PortC, ONE);
   else if(level == 2)
   	  DataSend2(PortC, TWO);
   else if(level == 3)
   	  DataSend2(PortC, THREE);
   else
      DataSend2(PortC, ZERO);
}

// A function which displays the remain life time that the player currently has on the 7 segment LED
static void LED_LifeDisplay(unsigned int lifenumber)
{
   if(lifenumber == 0)
   	  DataSend2(PortC, ZERO);
   else if(lifenumber == 1)
   	  DataSend2(PortC, ONE);
   else if(lifenumber == 2)
   	  DataSend2(PortC, TWO);
   else if(lifenumber == 3)
   	  DataSend2(PortC, THREE);
   else if(lifenumber == 4)
   	  DataSend2(PortC, FOUR);
   else if(lifenumber == 5)
   	  DataSend2(PortC, FIVE);
   else if(lifenumber == 6)
   	  DataSend2(PortC, SIX);
   else if(lifenumber == 7)
   	  DataSend2(PortC, SEVEN);
   else if(lifenumber == 8)
   	  DataSend2(PortC, EIGHT);
   else if(lifenumber == 9)
   	  DataSend2(PortC, NINE);
   else
      DataSend2(PortC, NINE);
}
 

// A function which transfer data from C32 to the 2nd P8255's output(To control three 7-segments LEDs) via the main P8255.
static void DataSend2(unsigned char portNumber, unsigned char data)
{
    unsigned char temp;
    
    if(portNumber == PortA) // means port A : The upper digit of game time LEDs.
    {
        //temp = data | 0x80; // 1000 0000 
        DataSend(PortB, temp); // (not)WR = 1
        delay_ms_times(1);
        
        //temp = current_PortC_value + 0x00;  // A1:0 A0:0
        current_extention2_addr = 0x00; 
        
          
        DataSend(PortC, current_PortC_value);
        delay_ms_times(1);  
        
              
        temp = data & 0x7F;	  // 0111 1111
        DataSend(PortB, temp); // (not)WR = 0
        delay_ms_times(2);
        
        temp = data | 0x80; // 1000 0000 
        DataSend(PortB, temp); // (not)WR = 1         
    } 
    else if(portNumber == PortB) // means port B : The lower digit of game time LEDs.
    {
        //temp = data | 0x80; // 1000 0000 
        DataSend(PortB, temp); // (not)WR = 1
        delay_ms_times(1);
        
        //temp = current_PortC_value + 0x40;  // A1:0 A0:1
        current_extention2_addr = 0x40; 
        
            
        DataSend(PortC, current_PortC_value);
        delay_ms_times(1);  
        
              
        temp = data & 0x7F;	  // 0111 1111
        DataSend(PortB, temp); // (not)WR = 0
        delay_ms_times(2);
        
        temp = data | 0x80; // 1000 0000 
        DataSend(PortB, temp); // (not)WR = 1         
    }
    else // means port C :  Game level and Life time displaying LED.
    {
        //temp = data | 0x80; // 1000 0000 
        DataSend(PortB, temp); // (not)WR = 1
        delay_ms_times(1);
        
        //temp = current_PortC_value + 0x80;  // A1:1 A0:0
        current_extention2_addr = 0x80; 
        
            
        DataSend(PortC, current_PortC_value);
        delay_ms_times(1);  
        
              
        temp = data & 0x7F;	  // 0111 1111
        DataSend(PortB, temp); // (not)WR = 0
        delay_ms_times(2);
        
        temp = data | 0x80; // 1000 0000 
        DataSend(PortB, temp); // (not)WR = 1         
    }
}


// A function which sends data to LCD or actuators via the main P8255( A port extension chip )
static void DataSend(unsigned char portNumber, unsigned char data)
{
    unsigned char temp;
    
    if(portNumber == PortA) // means port A
    {
        DDRAD = 0xFF;   // All outputs for data sending
        
        //--------------------------------------------------                
        //PTM = PTM | 0x10;   // 0001 0000, (not)cs = 1 
        PTM = PTM | 0x08;   // 0000 1000, (not)RD = 1
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
          //delay_ms_times(1);
        
        PTAD = 0xFF ^ data;
        if( (data & 0x80) == 0x80 ) // if the MSB is high
            PTM = PTM & 0xDF;   // and with 1101 1111
        else	  // if the MSB is low
            PTM = PTM | 0x20;   // or with 0010 0000
            
        PTM = PTM & 0xFC;   // 1111 1100, A1 A0: 0 0
           
        
        PTM = PTM & 0xFB;   // 1111 1011, (not)WR = 0
        delay_ms_times(1);
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
    }
    else if(portNumber == PortB) // means port B
    {
        DDRAD = 0xFF;   // All outputs for data sending
        
        //--------------------------------------------------                
        //PTM = PTM | 0x10;   // 0001 0000, (not)cs = 1 
        PTM = PTM | 0x08;   // 0000 1000, (not)RD = 1
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
        //delay_ms_times(1);
        
        PTAD = data;
        if( (data & 0x80) == 0x80 ) // if the MSB is high
            PTM = PTM | 0x20;   // or with 0010 0000
        else	  // if the MSB is low
            PTM = PTM & 0xDF;   // and with 1101 1111
            
        PTM = PTM & 0xFD;   // 1111 1101, A1: 0
        PTM = PTM | 0x01;   // 0000 0001, A0: 1
        
        PTM = PTM & 0xFB;   // 1111 1011, (not)WR = 0
        delay_ms_times(1);
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
        
    }
    else   // means upper bits of PortC
    {
        temp = (current_extention2_addr & 0xC0) | data;
        current_PortC_value = data;  
        
        DDRAD = 0xFF;   // All outputs for data sending
        
        //--------------------------------------------------                
        PTM = PTM | 0x08;   // 0000 1000, (not)RD = 1
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
                
        PTAD = temp;
        if( (temp & 0x80) == 0x80 ) // if the MSB is high
            PTM = PTM | 0x20;   // or with 0010 0000
        else	  // if the MSB is low
            PTM = PTM & 0xDF;   // and with 1101 1111
        
        PTM = PTM | 0x02;   // 0000 0010, A1: 1
        PTM = PTM & 0xFE;   // 1111 1110, A0: 0
       
        PTM = PTM & 0xFB;   // 1111 1011, (not)WR = 0
        delay_ms_times(1);
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
       
    }
}



// A function which receives data from the portC of the main P8255 chip.
static unsigned char DataReceive(void)
{
        unsigned char temp = 0x00;
        
        DDRAD = DDRAD & 0xF0;   // Set lower bits as inputs for data receiving
        
        //--------------------------------------------------                
        //PTM = PTM | 0x10;   // 0001 0000, (not)cs = 1 
        PTM = PTM | 0x04;   // 0000 0100, (not)WR = 1
        PTM = PTM | 0x08;   // 0000 1000, (not)RD = 1
        //delay_ms_times(1);
        PTM = PTM | 0x02;   // 0000 0010, A1: 1
        PTM = PTM & 0xFE;   // 1111 1110, A0: 0
        //PTM = PTM & 0xEF;   // 1110 11110, (not)cs = 0 
        //delay_ms_times(1);
        
        PTM = PTM & 0xF7;   // 1111 0111, (not)RD = 0
        
        delay_ms_times(1);
        temp = PTAD & 0x0F;
                
        
        PTM = PTM | 0x08;   // 0000 1000, (not)RD = 1
        //delay_ms_times(1);
        //PTM = PTM | 0x10;   // 0001 0000, (not)cs = 1  
    
        return temp;
}


// Coin detection fucntion. 
static void CoinDetecting(void)
{	
   int checking1 = 0, checking2 = 0;

   while( GetSignal(COIN) == 0 ) checking1 = 1;
   
   while( GetSignal(COIN) == 1 ) checking2 = 1;
   
   if(  checking1 == 1 &&  checking2 == 1)
      CurrentCoinNumber++ ;
}


// Game START detection function.
static void StartDetecting(void)
{
   /*int checking1 = 0, checking2 = 0;

   while( GetSignal(level_SW) == 0 ) checking1 = 1;
   
   while( GetSignal(level_SW) == 1 ) checking2 = 1;
   
   if(  checking1 == 1 &&  checking2 == 1)
     StartSignal = 1;
    */
        
    if( GetSignal(level_SW) == 1 )
        StartSignal = 1;
}


// Wire detection funciton.
static void WireDetecting(void)
{
   int checking1 = 0, checking2 = 0;
   
   DataReceive();
   //DataReceive();

   while( GetSignal(WireTouch) == 0 ) 
   {
        if((DataReceive() & 0x01) == 0x01)   // complete!
            return;
        
        checking1 = 1;
   }
   while( GetSignal(WireTouch) == 1 ) checking2 = 1;
   
   if(  checking1 == 1 &&  checking2 == 1)
     FailureSignal = 1;
}


// SWAG despense function.
static void SWAG_dispense(int duration)
{
    PWMS12_SetDuty(53, PWMS12_CHAN2);
    delay_ms_times(duration);
    PWMS12_SetDuty(23, PWMS12_CHAN2);
    delay_ms_times(1000); 
}


// A function directly controls two spining motors and the vibrating motor of the game stick.
static void Motor_Vibrator_Control(int motor_mode, int motor_speed, int vibrator_on)
{
    if(motor_mode==0) //oppposite direction
    {
        if(vibrator_on == 0) // vibrator off
        {
            DataSend(PortC, 0x00);
           
        } else
            DataSend(PortC, 0x10);
        
         PWMS12_SetDuty(motor_speed, PWMS12_CHAN0);
        
    }else   // same direction
    {
        if(vibrator_on == 0) // vibrator off
        {
            DataSend(PortC, 0x20);
           
        } else
            DataSend(PortC, 0x30);
        
         PWMS12_SetDuty(motor_speed, PWMS12_CHAN0);
    }
}
    


// A function which makes a sound of game-ending using the buzzer.
static void EndingSound(int mode)
{
    if(mode == 1)// success
    {
       PWMS12_SetDuty(50, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(60, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(70, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(80, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(90, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
    } 
    else   // failed
    {
       PWMS12_SetDuty(70, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(60, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(50, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(40, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
	   
	   PWMS12_SetDuty(30, PWMS12_CHAN1);   // buzzer on
       delay_ms_times(1000);
       PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off 
	   delay_ms_times(500);
    }
	 
}

// A funciton which provides 'sensors warm-up' to guarantee more stable system.
static void warming_up(void)
{
    int i=0;
    
    for(i=0;i<100;i++)
    {
        DataReceive();
    }
}




void main(void)
{
  unsigned Current_State = RESET_STATE;	// When the program start, the first state is 'RESET_STATE'.
  int i=0;
    
  
  while(1)
  {
      switch(Current_State)
      {
	// Behavior of RESET_STATE
        case RESET_STATE:
            PORTInit();	
            LCDInit();
            
	    // Initially set all 7-segments LEDs as zero.
            LED_LevelDisplay(0);	
            DataSend2(PortA, ZERO);
            DataSend2(PortB, ZERO);
            
	    // LCD instructions
            LCDputString(0, "Welcome to Wire Game     ");
            delay_ms_times(2000);
            LCDputString(1, "Please insert Coin      ");
            
	    // Waiting for coin detection.
            CoinDetecting();
            
            // When coin detected, set the new current state for the next state.
	    if( CurrentCoinNumber != 0 )
                Current_State = COIN_STATE;
            
            else
                ;
            
            break;
        
        // Behavior of COIN_STATE
	case COIN_STATE:
            // Instructions using LCD
	    LCDputString(5, "Thank you!                 ");
            delay_ms_times(2000);
            LCDputString(2, "Select the level,            ");
            delay_ms_times(2000);
            LCDputString(0, "And Push the Button         ");
            delay_ms_times(2000);
            
            //level selection by potentiometer input and display the result to LCD and 7-segment LED in real time.
            while( StartSignal == 0)
            {
                if(ADS12_ReadADPin(7) <1008) // lower about 2V
                {
                    LCDputString(0,"1.LOW LEVEL GAME              ");
                    LED_LevelDisplay(1);
                    LimitTime = 99;	// Low level's play time is 99 seconds. 
                    LifeNumber = 15;	// Low level's life time is 15. (decrease by one every wire touching)
                    LED_TimeDisplay(LimitTime);
                }
                else  if(ADS12_ReadADPin(7) < 1017) // lower about 3.5V
                {
                    LCDputString(0,"2.MED LEVEL GAME              ");
                    LED_LevelDisplay(2);
                    LimitTime = 85;	// Low level's play time is 85 seconds. 
                    LifeNumber = 11;	// Low level's life time is 11. (decrease by one every wire touching)
                    LED_TimeDisplay(LimitTime);
                }
                else
                {
                    LCDputString(0,"3.HIGH LEVEL GAME              ");
                    LED_LevelDisplay(3);
                    LimitTime = 75;	// Low level's play time is 75 seconds. 
                    LifeNumber = 7;	// Low level's life time is 7. (decrease by one every wire touching)
                    LED_TimeDisplay(LimitTime);
                }
                
                StartDetecting();	// check whether the start button is pushed or not
            }
            
	    // when the start button is pushed
            if( StartSignal != 0 )
            {
		// set the new current state by player's level selection
                if(ADS12_ReadADPin(7) <1008) // lower about 2V
                    Current_State = LOW_PLAY_STATE;
                else  if(ADS12_ReadADPin(7) < 1017) // lower about 3.5V
                    Current_State = MED_PLAY_STATE;
                else
                    Current_State = HIGH_PLAY_STATE; 
            }
            else
                ; 
            
            
            break;
        
	// Behavior of LOW_PLAY_STATE (low level playing)	 
        case LOW_PLAY_STATE:
        
            warming_up();
        	
	    // Make two DC motors run in opposite direction and the vibration motor off. 
            Motor_Vibrator_Control(0, 3, 0);
            LCDputString(0,"Enjoy Wire Game!         ");
            
            i = LimitTime;
            
	    // decrease the playing time and check the sensor inputs simutaneously(includes the wire touching).
            do
            {
                LED_LifeDisplay(LifeNumber);
                LED_TimeDisplay(i);
                delay_with_service(1000);
                
		// Change the LCD display by the current playing region(For more fun!!!).
                if((i%4 == 0) || (i%4 == 1))
                {
                    if(current_region == 1)
                       LCDputString(0,"Keep Going !!!      ");
                    else if(current_region == 2)
                       LCDputString(0,"Very Good job !!!      ");
                    else if(current_region == 3)
                       LCDputString(0,"Be cautious the wheels !!!    ");
                    else if(current_region == 4)
                       FailureSignal = 2;   // means success
                    else
                    ;
                }else
                {
                    if(current_region == 1)					 
                       LCDputString(0,"THIS IS THE REGION 1      ");
                    else if(current_region == 2)
                       LCDputString(0,"THIS IS THE REGION 2      ");
                    else if(current_region == 3)
                       LCDputString(0,"THIS IS THE REGION 3      ");
                    else if(current_region == 4)
                       FailureSignal = 2;   // means success
                    else
                    ;
                }
                               
                i--;
                
            }while((i>=0) && (FailureSignal == 0));
            
            
            if( FailureSignal == 1 )	// Game over by over touching
            {
                Current_State = FAILURE_STATE;
                LCDputString(0,"Wire Touched !!!       ");            
            }
            else if( FailureSignal == 2 )   // This means success!!!!
                Current_State = SUCCESS_STATE;
            
            else		// Game over by over time
            {
                Current_State = FAILURE_STATE;
                LCDputString(0,"Time Expired !!!       ");            
            }
            
            break;
       
       // Behavior of MED_PLAY_STATE (Median level playing)     	 
       case MED_PLAY_STATE:
        				         
            warming_up();
            // Make two DC motors run in the same direction and the vibration motor off. 
            Motor_Vibrator_Control(1, 4, 0);
            LCDputString(0,"Enjoy Wire Game!         ");
            
            //LimitTime = 99;
            i = LimitTime;
            
	    // decrease the playing time and check the sensor inputs simutaneously(includes the wire touching).
            do
            {
                LED_LifeDisplay(LifeNumber);
                LED_TimeDisplay(i);
                delay_with_service(1000);
                
                // Change the LCD display by the current playing region(For more fun!!!).
		if((i%4 == 0) || (i%4 == 1))
                {
                    if(current_region == 1)
                       LCDputString(0,"Keep Going !!!      ");
                    else if(current_region == 2)		   
                       LCDputString(0,"Very Good job !!!      ");
                    else if(current_region == 3)
                       LCDputString(0,"Be cautious the wheels !!!     ");
                    else if(current_region == 4)
                       FailureSignal = 2;   // means success
                    else
                    ;
                }else
                {
                    if(current_region == 1)
                       LCDputString(0,"THIS IS THE REGION 1    ");
                    else if(current_region == 2)
                       LCDputString(0,"THIS IS THE REGION 2   ");
                    else if(current_region == 3)
                       LCDputString(0,"THIS IS THE REGION 3   ");
                    else if(current_region == 4)
                       FailureSignal = 2;   // means success
                    else
                    ;
                }
                               
                i--;
                
            }while((i>=0) && (FailureSignal == 0));
            
            
            if( FailureSignal == 1 )	// Game over by over touching
            {
                Current_State = FAILURE_STATE;
                LCDputString(0,"Wire Touched !!!       ");            
            }
            else if( FailureSignal == 2 )   // This means success!!!!
                Current_State = SUCCESS_STATE;
            
            else		// Game over by over time
            {
                Current_State = FAILURE_STATE;
                LCDputString(0,"Time Expired !!!       ");            
            }
            
            break;
            
       case HIGH_PLAY_STATE:
        
            warming_up();
            
	    // Make two DC motors run in the same direction and the vibration motor on. 
            Motor_Vibrator_Control(1, 4, 1);
            LCDputString(0,"Enjoy Wire Game!         ");
            
            i = LimitTime;
            
	    // decrease the playing time and check the sensor inputs simutaneously(includes the wire touching).
            do
            {
                LED_LifeDisplay(LifeNumber);
                LED_TimeDisplay(i);
                delay_with_service(1000);
                
		// Change the LCD display by the current playing region(For more fun!!!).
                if((i%4 == 0) || (i%4 == 1))
                {
                    if(current_region == 1)
                       LCDputString(0,"Keep Going !!!      ");
                    else if(current_region == 2)
                       LCDputString(0,"Very Good job !!!      ");
                    else if(current_region == 3)
                       LCDputString(0,"Be cautious the wheels !!!   ");
                    else if(current_region == 4)
                       FailureSignal = 2;   // means success
                    else
                    ;
                }else
                {
                    if(current_region == 1)
                       LCDputString(0,"THIS IS THE REGION 1   ");
                    else if(current_region == 2)
                       LCDputString(0,"THIS IS THE REGION 2   ");
                    else if(current_region == 3)
                       LCDputString(0,"THIS IS THE REGION 3   ");
                    else if(current_region == 4)
                       FailureSignal = 2;   // means success
                    else
                    ;
                }
                
                i--;
                
            }while((i>=0) && (FailureSignal == 0));
            
            
            if( FailureSignal == 1 )	// Game over by over touching
            {
                Current_State = FAILURE_STATE;
                LCDputString(0,"Wire Touched !!!       ");            
            }
            else if( FailureSignal == 2 )   // This means success!!!!
                Current_State = SUCCESS_STATE;
            
            else		// Game over by over time
            {
                Current_State = FAILURE_STATE;
                LCDputString(0,"Time Expired !!!       ");            
            }
            
            break;

	// Game ending after failure.
        case FAILURE_STATE:
            
            PTM = PTM | 0x10; // 0001 0000 Turn off the solenoid. 
            LCDputString(0, "Sorry.You failed.T_T      ");
            
	    // Turn off the all actuators
	    Motor_Vibrator_Control(0, 0, 0);
            
	    // Buzzer sounds to indicate the failure
            PWMS12_SetDuty(80, PWMS12_CHAN1);   // buzzer on
            delay_ms_times(2000);
            PWMS12_SetDuty(0, PWMS12_CHAN1);    // buzzer off
            
            // controls the servo motor to dispense SWAG
            SWAG_dispense(400);
           
            delay_ms_times(3000);                  
            
	    // Return to the basic state, the reset state			
            Current_State = RESET_STATE;
            break;
        
        // Game ending after success.
        case SUCCESS_STATE:
             
            PTM = PTM | 0x10; // 0001 0000 Turn off the solenoid.  
            LCDputString(0, "Congratulation! *^^*        ");
            
	    // Turn off the all actuators
	    Motor_Vibrator_Control(0, 0, 0);
            
	    // Buzzer sounds to congratulate the success!
            EndingSound(1);
            
            delay_ms_times(2000);
            // controls the servo motor to dispense SWAG
	    SWAG_dispense(700);
            SWAG_dispense(700);
            SWAG_dispense(700);
            delay_ms_times(3000);    
        
            // Return to the basic state, the reset state           
            Current_State = RESET_STATE;
            break;
      }
  } 
    

}



